# -*- python -*-
# $RCSfile: pixelinfoGUI.py,v $
# $Revision: 1.41 $
# $Author: langer $
# $Date: 2007/05/17 19:10:13 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import pixelgroup
from oof2.SWIG.common import switchboard
from oof2.SWIG.common import ooferror
from oof2.common import debug
from oof2.common import guitop
from oof2.common import mainthread
from oof2.common import primitives
from oof2.common import subthread
from oof2.common.IO import pixelinfo
from oof2.common.IO.GUI import gtklogger
from oof2.common.IO.GUI import gtkutils
from oof2.common.IO.GUI import mousehandler
from oof2.common.IO.GUI import toolboxGUI
import gtk
import oof2.common.microstructure

#####################

# PixelInfoPlugIn classes need the following things:
#   * ordering, at the class level
#   * nrows, at the class level
#   * A constructor with arguments (GtkTable, row) which inserts nrows
#     of widgets into the table starting at the given row.
#   * A function update(toolbox, iPoint) that updates the widgets
#   * A function close() that's called when the toolbox is closed.
#   * A function nonsense() that's called when the mouse click isn't sensible.
#   * A function clear() that's called when the clear button is pressed.

# Here's a nearly useless baseclass:
class PixelInfoPlugIn:
    def __init__(self, toolbox):
        self.toolbox = toolbox
    def close(self):
        del self.toolbox
    def update(self, point):
        pass
    def nonsense(self):
        pass
    def clear(self):
        pass
    

plugInClasses = []

def pluginsorter(a,b):
    if a.ordering < b.ordering: return -1
    if a.ordering > b.ordering: return 1
    return 0

def registerPlugInClass(plugin):
    plugInClasses.append(plugin)
    plugInClasses.sort(pluginsorter)
    switchboard.notify('new pixelinfo plugin')

######################

class PixelInfoToolboxGUI(toolboxGUI.GfxToolbox, mousehandler.MouseHandler):
    def __init__(self, pixelinfotoolbox):
        debug.mainthreadTest()
        toolboxGUI.GfxToolbox.__init__(self, "Pixel Info", pixelinfotoolbox)
        mainbox = gtk.VBox()
        self.gtk.add(mainbox)

        self.table = gtk.Table(rows=3, columns=2)
        mainbox.pack_start(self.table, expand=0, fill=0)
        
        label = gtk.Label('x=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 0,1, xpadding=5, xoptions=gtk.FILL)
        self.xtext = gtk.Entry()
        gtklogger.setWidgetName(self.xtext, "X")
        self.xtext.set_size_request(10*guitop.top().digitsize, -1)
        self.table.attach(self.xtext, 1,2, 0,1,
                          xpadding=5, xoptions=gtk.EXPAND|gtk.FILL)
        label = gtk.Label('y=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 1,2, xpadding=5, xoptions=gtk.FILL)
        self.ytext = gtk.Entry()
        gtklogger.setWidgetName(self.ytext, "Y")
        self.ytext.set_size_request(10*guitop.top().digitsize, -1)
        self.table.attach(self.ytext, 1,2, 1,2,
                          xpadding=5, xoptions=gtk.EXPAND|gtk.FILL)


        self.xtsignal = gtklogger.connect(self.xtext, 'changed',
                                          self.pointChanged)
        self.ytsignal = gtklogger.connect(self.ytext, 'changed',
                                          self.pointChanged)

        box = gtk.HBox(homogeneous=True, spacing=2)
        self.updatebutton = gtkutils.stockButton(gtk.STOCK_REFRESH, 'Update')
        box.pack_start(self.updatebutton, expand=1, fill=1)
        gtklogger.setWidgetName(self.updatebutton, "Update")
        gtklogger.connect(self.updatebutton, 'clicked', self.updateButtonCB)
        self.clearbutton = gtkutils.stockButton(gtk.STOCK_CLEAR, 'Clear')
        box.pack_start(self.clearbutton, expand=1, fill=1)
        gtklogger.setWidgetName(self.clearbutton, "Clear")
        gtklogger.connect(self.clearbutton, 'clicked', self.clearButtonCB)
        self.table.attach(box, 0,2, 2,3,
                          xpadding=5, xoptions=gtk.EXPAND|gtk.FILL, yoptions=0)

        self.updatebutton.set_sensitive(0)
        self.clearbutton.set_sensitive(0)
        self.buildGUI()
        
        self.sbcallbacks = [
            switchboard.requestCallbackMain(pixelinfotoolbox,
                                            self.update),
            switchboard.requestCallbackMain('new pixelinfo plugin',
                                            self.buildGUI),
            switchboard.requestCallbackMain((self.gfxwindow(),
                                             'layers changed'), self.update)
            ]

    def buildGUI(self):
        debug.mainthreadTest()
        self.table.resize(rows=4, columns=2)
        row = 4
        self.plugins = []
        for pluginclass in plugInClasses:
            self.table.resize(row + 1 + pluginclass.nrows, 2)
            # A gtk or pygtk bug requires xpadding to be set
            # explicitly if ypadding is set here.
            self.table.attach(gtk.HSeparator(), 0,2, row,row+1,
                              xoptions=gtk.EXPAND|gtk.FILL,
                              xpadding=0, ypadding=3)
            self.plugins.append(pluginclass(self, self.table, row+1))
            row += pluginclass.nrows+1

    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.gfxwindow().setMouseHandler(self)

    def deactivate(self):
        toolboxGUI.GfxToolbox.deactivate(self)
        self.gfxwindow().removeMouseHandler()

    def close(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        for plugin in self.plugins:
            plugin.close()
        self.plugins = []

    def acceptEvent(self, eventtype):
        return eventtype == 'up'

    # It's possible to click on the background of the canvas, but
    # outside the image/microstructure.  In this case, do nothing --
    # the behavior is then the same as if you click outside the drawn
    # area.
    def up(self, x, y, shift, ctrl):
        msOrImage = self.gfxwindow().topmost('Microstructure', 'Image')
        if msOrImage:
            where = msOrImage.pixelFromPoint(primitives.Point(x,y))
            if msOrImage.pixelInBounds(where):
                self.toolbox.menu.Query(x=where.x, y=where.y)
        else:
            for plugin in self.plugins:
                plugin.nonsense();
            

    def currentPixel(self):
        return self.toolbox.currentPixel()
    
    def update(self):
        debug.mainthreadTest()
        self.xtsignal.block()
        self.ytsignal.block()
        try:
            where = self.currentPixel()

            if where is not None:
                self.clearbutton.set_sensitive(1)
            else:
                self.xtext.set_text('')
                self.ytext.set_text('')
                self.clearbutton.set_sensitive(0)
                for plugin in self.plugins:
                    plugin.clear()

            msOrImage = self.gfxwindow().topmost('Microstructure', 'Image')
            if where is not None and msOrImage is not None:
                self.xtext.set_text(`where.x`)
                self.ytext.set_text(`where.y`)
                size = msOrImage.sizeInPixels() # might be ICoord or iPoint
                if 0 <= where.x < size[0] and 0 <= where.y < size[1]:
                    for plugin in self.plugins:
                        plugin.update(where)
                else:
                    for plugin in self.plugins:
                        plugin.nonsense()
            else:
                for plugin in self.plugins:
                    plugin.nonsense()

                self.updatebutton.set_sensitive(0)
        finally:
            self.xtsignal.unblock()
            self.ytsignal.unblock()

        gtklogger.checkpoint(self.gfxwindow().name + " " +
                             self._name + " updated")

    def updateButtonCB(self, button):
        debug.mainthreadTest()
        self.updatebutton.set_sensitive(0)
        x = int(self.xtext.get_text())
        y = int(self.ytext.get_text())
        msOrImage = self.gfxwindow().topmost('Microstructure', 'Image')
        size = msOrImage.sizeInPixels() # might be ICoord or iPoint
        if x < 0 or x >= size[0] or y < 0 or y >= size[1]: # illegal size
            self.update()               # restore original values
            return
        self.toolbox.menu.Query(x=x, y=y)

    def clearButtonCB(self, button):
        self.toolbox.menu.Clear()

    def pointChanged(self, *args):
        debug.mainthreadTest()
        self.updatebutton.set_sensitive(1)

    def findMicrostructure(self):
        return self.toolbox.findMicrostructure()

def _makeGUI(self):
    return PixelInfoToolboxGUI(self)

pixelinfo.PixelInfoToolbox.makeGUI = _makeGUI

####################################

class MicrostructurePlugIn(PixelInfoPlugIn):
    ordering = 2
    nrows = 2
    def __init__(self, toolbox, table, row):
        debug.mainthreadTest()
        PixelInfoPlugIn.__init__(self, toolbox)
        label = gtk.Label('microstructure=')
        label.set_alignment(1.0, 0.5)
        table.attach(label, 0,1, row,row+1, xpadding=5, xoptions=gtk.FILL)
        self.microtext = gtk.Entry()
        gtklogger.setWidgetName(self.microtext,'MSText')
        self.microtext.set_size_request(12*guitop.top().charsize, -1)
        self.microtext.set_editable(0)
        table.attach(self.microtext, 1,2, row,row+1,
                     xpadding=5, xoptions=gtk.EXPAND|gtk.FILL)

        label = gtk.Label('pixel groups=')
        label.set_alignment(1.0, 0.5)
        table.attach(label, 0,1, row+1,row+2, xpadding=5, xoptions=gtk.FILL)
        self.grouplist = gtk.TextView()
        self.grouplist.set_size_request(12*guitop.top().charsize, -1)
        self.grouplist.set_editable(False)
        self.grouplist.set_cursor_visible(False)
        self.grouplist.set_wrap_mode(gtk.WRAP_WORD)
        gtklogger.setWidgetName(self.grouplist, 'Group view')
        scroll = gtk.ScrolledWindow()
        gtklogger.logScrollBars(scroll, "MSScroll")
        scroll.set_shadow_type(gtk.SHADOW_IN)
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroll.add(self.grouplist)
        table.attach(scroll, 1,2, row+1,row+2,
                     xpadding=5, xoptions=gtk.EXPAND|gtk.FILL)

        self.sbcallbacks = [
            switchboard.requestCallbackMain('changed pixel group',
                                            self.grpchanged),
            switchboard.requestCallbackMain('changed pixel groups',
                                            self.grpschngd),
            switchboard.requestCallbackMain('destroy pixel group',
                                            self.grpdestroy),
            switchboard.requestCallbackMain('renamed pixel group',
                                            self.grprenamed)
            ]

    def close(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        PixelInfoPlugIn.close(self)

    def clear(self):
        debug.mainthreadTest()
        self.microtext.set_text("")
        self.grouplist.get_buffer().set_text("")
        
    def update(self, where):
        subthread.execute(self.update_thread, (where,))
    def update_thread(self, where):
        debug.subthreadTest()
        microstructure = self.toolbox.findMicrostructure()
        if microstructure and where is not None:
            mscntxt = oof2.common.microstructure.getMSContextFromMS(
                microstructure)
            msname = microstructure.name()
            mscntxt.begin_reading()
            try:
                names = pixelgroup.pixelGroupNames(microstructure, where)
            finally:
                mscntxt.end_reading()
            grpnames = '\n'.join(names)
        else:
            msname = '(No microstructure)'
            grpnames = ''
        mainthread.runBlock(self.reallyupdate, (msname, grpnames))
    def reallyupdate(self, msname, grpnames):
        debug.mainthreadTest()
        self.microtext.set_text(msname)
        self.grouplist.get_buffer().set_text(grpnames)
##        self.grouplist.show_all()

    def nonsense(self):
        debug.mainthreadTest()
        self.grouplist.get_buffer().set_text('')
        self.microtext.set_text('???')
    def grpchanged(self, group, ms_name):
        microstructure = self.toolbox.findMicrostructure()
        if microstructure and microstructure.name() == ms_name:
            if group.name() in microstructure.groupNames():
                self.update(self.toolbox.currentPixel())
    def grpschngd(self, ms_name):
        microstructure = self.toolbox.findMicrostructure()
        if microstructure and microstructure.name() == ms_name:
            self.update(self.toolbox.currentPixel())
    def grpdestroy(self, group, ms_name):
        microstructure = self.toolbox.findMicrostructure()
        if microstructure and microstructure.name() == ms_name:
            self.update(self.toolbox.currentPixel())                
    def grprenamed(self, group, oldname, newname):
        microstructure = self.toolbox.findMicrostructure()
        if microstructure:
            if newname in microstructure.groupNames():
                self.update(self.toolbox.currentPixel())

registerPlugInClass(MicrostructurePlugIn)
