// -*- C++ -*-
// $RCSfile: compoundsubproblem.C,v $
// $Revision: 1.4 $
// $Author: langer $
// $Date: 2007/01/29 00:10:29 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include "engine/compoundsubproblem.h"

BinarySubproblemPredicate::BinarySubproblemPredicate(CSubProblem *a,
						     CSubProblem *b)
  : subA(a),
    subB(b)
{}

bool UnionSBPredicate::operator()(const FEMesh *mesh, const Element *el) {
  return subA->contains(el) || subB->contains(el);
}

bool IntersectionSBPredicate::operator()(const FEMesh *mesh, const Element *el){
  return subA->contains(el) && subB->contains(el);
}

bool XorSBPredicate::operator()(const FEMesh *mesh, const Element *el) {
  // bitwise xor is ok since CSubProblem::contains returns a bool.
  return subA->contains(el) ^ subB->contains(el);
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

ComplementSBPredicate::ComplementSBPredicate(CSubProblem *comp)
  : complement(comp)
{}

bool ComplementSBPredicate::operator()(const FEMesh *mesh, const Element *el) {
  return not complement->contains(el);
}
