// -*- C++ -*-
// $RCSfile: fluxnormal.C,v $
// $Revision: 1.3 $
// $Author: reida $
// $Date: 2004/07/07 13:32:41 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include "fluxnormal.h"

// Support routines for the generic access method for the FluxNormal
// derived classes.  These are so that you can still query the 
// values even if you're not sure which type you've gotten.


double VectorFluxNormal::operator[](int n) const {
  if(n==0) 
    return val_;
  else
    return 0.0;  // Throw a range error or something.
}

double SymTensorFluxNormal::operator[](int n) const {
  switch(n) {
  case 0: 
    return x;
    break;
  case 1:
    return y;
    break;
  default:
    return 0.0; // Throw range error.
  }
}

// n is the boundary frame's x-axis, given in lab-frame coordinates.
// Use this to transform the given x and y, given in boundary-frame
// coordinates, into lab coordinates.
void SymTensorFluxNormal::transform(const Coord &n) {
  double normal_frame_x = x;
  double normal_frame_y = y;
  x=n(0)*normal_frame_x-n(1)*normal_frame_y;
  y=n(0)*normal_frame_y+n(1)*normal_frame_x;
}
