// -*- C++ -*-
// $RCSfile: forcedensity.h,v $
// $Revision: 1.9 $
// $Author: langer $
// $Date: 2007/04/27 15:58:53 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#ifndef FORCEDENSITY_H
#define FORCEDENSITY_H

#include "engine/property.h"
#include <string>

class Cijkl;
class CSubProblem;
class Element;
class Flux;
class Material;
class FEMesh;
class Position;
class TwoVectorField;
class SymmetricTensorFlux;
class ElementNodeIterator;

class ForceDensity : public Property {
private:
  double gx, gy;
public:
  ForceDensity(PyObject *reg, const std::string &name, double x, double y);
  virtual ~ForceDensity() {}
  virtual void precompute(FEMesh*);
  virtual void fluxrhs(const FEMesh*, const Element*, const Flux*, FluxData*,
		       const MasterPosition&) const;
  virtual int integration_order(const CSubProblem*, const Element*) const;
  double fdensity_x() const { return gx; }
  double fdensity_y() const { return gy; }
  virtual bool is_symmetric(const CSubProblem*) const { return true; }
  virtual bool constant_in_space() const { return true; }
protected:
  TwoVectorField *displacement;
  SymmetricTensorFlux *stress_flux;
};

#endif
