// -*- C++ -*-
// $RCSfile: pyroelectricity.h,v $
// $Revision: 1.8 $
// $Author: langer $
// $Date: 2007/05/10 21:22:19 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#ifndef PYROELECTRICITY_H
#define PYROELECTRICITY_H

// Base class for PiezoElectricity Properties

#include "engine/property.h"
#include "engine/matvec.h"
#include "engine/property.h"
#include <string>

class Elasticity;
class PiezoElectricity;
class ThermalExpansion;
class Element;
class Flux;
class FluxData;
class Material;
class Position;
class TwoVectorField;
class OrientationPropBase;
class OutputVal;
class PropertyOutput;
class ScalarField;
class SymmetricTensorFlux;
class VectorFlux;

// Pyroelectricity is dependent on temperature, and contributes to the
// polarization flux.  The coefficient is the constant-strain modulus,
// and is a three-vector, and works in the obvious way.  Users may
// specify the constant-stress modulus.  In this case, the
// constant-strain modulus can be deduced from the constant-stress
// modulus by subtracting the product of the piezoelectric d tensor,
// the isothermal elastic constants, and the strain thermal expansion
// coefficient.  If any of these are zero, then the constant stress
// and constant strain pyroelectric moduli are equal.

class PyroElectricity : public Property {
protected:
  // Physical parameters.
  Elasticity *elasticity;	// pointer to this Material's Elasticity
  PiezoElectricity *piezoelectricity;
  ThermalExpansion *thermalexpansion;
  ScalarField *temperature;
  VectorFlux *total_polarization;
  OrientationPropBase *orientation;
  VECTOR_D modulus;
  VECTOR_D lab_modulus;  // Constant-stress modulus.
  double tzero;
  std::string coefficient_type;
  void set_effective_modulus(const FEMesh*, const Element*,
			     const MasterPosition&) const;
private:
  mutable VECTOR_D effective_modulus;
  mutable bool modulus_ok;
public:
  PyroElectricity(PyObject *registry, 
		  const std::string &nm,
		  double px, double py, double pz, double t0,
		  std::string *ctype);
  virtual ~PyroElectricity() {}
  virtual void cross_reference(Material*);
  virtual void precompute(FEMesh*);

  virtual void fluxmatrix(const FEMesh*, const Element*, 
			  const ElementFuncNodeIterator&,
			  const Flux*, FluxData*, const MasterPosition&) const;
  
  
  virtual void fluxrhs(const FEMesh*, const Element*,
		       const Flux*, FluxData*, const MasterPosition&) const;

  virtual void output(const FEMesh*, const Element*, const PropertyOutput*,
		      const MasterPosition&, OutputVal*) const;

  virtual bool constant_in_space() const { return true; }
  virtual int integration_order(const CSubProblem*, const Element*) const;

  virtual bool is_symmetric(const CSubProblem*) const;

};

#endif
