# -*- python -*-
# $RCSfile: revertmesh.py,v $
# $Revision: 1.9 $
# $Author: reida $
# $Date: 2007/04/23 15:01:47 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.common import registeredclass
from oof2.common.IO import parameter
from oof2.engine import errorestimator
from oof2.engine import meshmod
from oof2.engine import skeletonelement
from oof2.engine import refine
from oof2.SWIG.common import switchboard

import string

## TODO SUBPROBLEMS: Update all aspects of AdaptiveMeshRefinement for
## Subproblems.

# AdaptiveMeshRefinement
class RevertMesh(meshmod.MeshModification):
    def __init__(self, base_skeleton, init_field=1):
        self.base_skeleton = base_skeleton
        self.init_field = init_field
        
    def apply(self, meshcontext):
        old_femesh = meshcontext.femesh()  # the current one
        old_skel = meshcontext.getSkeleton()
        # Names of base skeletons are automatically generated and
        # users can't access it.
        # If there're 3 base skeletons, they'll be
        # "Base Skeleton 0", "Base Skeleton 1", "Base Skeleton 2".
        # The numbers in the name string match their positions in the buffer,
        # which makes retrieving the object simple.
        ## TODO: What if there are more than 10 skeletons?  This
        ## retrieval method is *too* simple.
        index = int(string.split(self.base_skeleton)[-1])
        # if 'Base Skeleton 0' => 0
        new_skel = meshcontext.skeleton_buffer.makeCurrent(index)
        realmaterial = skeletonelement.SkeletonElement.realmaterial
        new_femesh = new_skel.femesh(meshcontext.elementdict,
                                     realmaterial)
        new_femesh.set_rwlock(meshcontext.rwLock)
        
        meshcontext.setSkeleton(new_skel)  # replace old_skel

        # copy field status.  Only compound fields can be activated/defined.
        for field in meshcontext.all_compound_subproblem_fields():
            new_femesh.define_field(field)
            new_femesh.activate_field(field)
            if old_femesh.in_plane(field):
                field.set_in_plane(new_femesh, 1)
        
        # copy equation status
        for equation in meshcontext.all_equations():
            new_femesh.activate_equation(equation)

        # replace old_femesh
        meshcontext.setFemesh(new_femesh)

        # putting BC in new_Femesh
        for (name, bc) in meshcontext.allBoundaryConds():
            bc.boundary_obj=new_femesh.getBoundary(bc.boundary)
            new_femesh.boundaries[bc.boundary].addCondition(bc)
            
        if self.init_field:
            # Fields to be initialized from old to new.
            for field in meshcontext.all_fields():
                new_femesh.init_field(old_skel, old_femesh, field)
            ## transfer time index
            new_femesh.set_time(old_femesh.get_subproblem().current_time())
        # destroying old_femesh
        old_femesh.destroy()

        # time stamp update
        meshcontext.changed()


# Trivial parameter for skeletons of the mesh -- it will have
# a custom chooser widget.
class MeshSkeletonParameter(parameter.StringParameter):
    pass

registeredclass.Registration(
    'Revert Mesh',
    meshmod.MeshModification,
    RevertMesh,
    ordering=3,
    params=[
    MeshSkeletonParameter('base_skeleton', tip="Name of the base Skeleton."),
    parameter.BooleanParameter('init_field', 1, default=1,
                               tip="Initialize fields?")
    ],
    tip="Undo adaptive mesh refinement.",
    discussion="""<para>
    Refined &mesh; can be reverted back to its pre-refined stages, in case
    the refinement was not satisfactory. Field values (solution) may not be
    recovered properly in some cases.
    </para>"""
    )
