// -*- C++ -*-
// $RCSfile: orientmapproperty.C,v $
// $Revision: 1.1 $
// $Author: langer $
// $Date: 2007/05/17 19:10:14 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include "common/cmicrostructure.h"
#include "engine/corientation.h"
#include "engine/element.h"
#include "engine/femesh.h"
#include "engine/mastercoord.h"
#include "engine/ooferror.h"
#include "orientationmap/orientmapproperty.h"

OrientationMapProp::OrientationMapProp(PyObject *registry,
				       const std::string &name)
  : OrientationPropBase(registry, name)
{}

class OrientMapMeshData {
public:
  CMicrostructure *microstructure;
  OrientMap *odata;
  OrientMapMeshData(CMicrostructure *ms, OrientMap *orientmap)
    : microstructure(ms),
      odata(orientmap)
  {}
};

void OrientationMapProp::precompute(FEMesh *mesh) {
  CMicrostructure *microstructure = mesh->get_microstructure();
  OrientMap *orientmap = getOrientMap(microstructure->name());
  if(orientmap == 0)
    throw ErrUserError("Microstructure does not have an Orientation Map!");
  set_mesh_data(mesh, new OrientMapMeshData(microstructure, orientmap)); 
}

const COrientation *OrientationMapProp::orientation(const FEMesh *mesh,
						    const Element *element,
						    const MasterPosition &mpos)
  const
{
  OrientMapMeshData *meshdata =
    static_cast<OrientMapMeshData*>(get_mesh_data(mesh));
  Coord where = element->from_master(mpos);
  return &meshdata->odata->angle(
			    meshdata->microstructure->pixelFromPoint(where));
}

void OrientationMapProp::clear_mesh_data(FEMesh *mesh, void *data) const {
  delete static_cast<OrientMapMeshData*>(data);
}

const COrientation *OrientationMapProp::orientation() const {
  static COrientABG dummy(0.0, 0.0, 0.0);
  return &dummy;
}
