# -*- python -*-
# $RCSfile: boundary_condition_test.py,v $
# $Revision: 1.4 $
# $Author: langer $
# $Date: 2007/03/28 15:02:26 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

# Tests for the machinery that handles boundary condition
# intersections, floatBC promotion, and enabling and disabling of
# boundary conditions.

import unittest, os

class OOF_BCTest(unittest.TestCase):
    def setUp(self):
        OOF.File.Load.Data(
            filename=os.path.join("bc_data", "periodic_test_mesh"))


    def SimpleSolve(self):
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
        OOF.File.Load.Data(filename=os.path.join("bc_data", "simple_solution_test"))
        from oof2.engine import mesh
        saved = mesh.meshes["saved_data:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)
        OOF.Microstructure.Delete(microstructure="saved_data")
 

    # periodic in one direction
    def Periodic1(self):
        OOF.Mesh.Boundary_Conditions.New(name='bc<3>', mesh='bc_test:skeleton:mesh',
                                         condition=PeriodicBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',boundary='top bottom'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='periodic_1')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "periodic_1"),
##                            mode="w",format="ascii",mesh="periodic_1:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='periodic_1', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "periodic_1"))
        from oof2.engine import mesh
        saved = mesh.meshes["periodic_1:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)
        OOF.Microstructure.Delete(microstructure="periodic_1")


    # periodic in two directions
    def Periodic2(self):
        OOF.Mesh.Boundary_Conditions.New(name='bc<3>', mesh='bc_test:skeleton:mesh',
                                         condition=PeriodicBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',boundary='top bottom'))
        OOF.Mesh.Boundary_Conditions.New(name='bc<4>', mesh='bc_test:skeleton:mesh',
                                         condition=PeriodicBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',boundary='left right'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='periodic_2')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "periodic_2"),
##                            mode="w",format="ascii",mesh="periodic_2:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='periodic_2', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "periodic_2"))
        from oof2.engine import mesh
        saved = mesh.meshes["periodic_2:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)
        OOF.Microstructure.Delete(microstructure="periodic_2")


    # float intersecting float
    def TwoFloats(self):
        OOF.Mesh.Boundary_Conditions.New(name='bc<3>', mesh='bc_test:skeleton:mesh',
                                         condition=FloatBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='0'),boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(name='bc<4>', mesh='bc_test:skeleton:mesh',
                                         condition=FloatBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='alpha'),boundary='left'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='twofloats')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "twofloats"),
##                            mode="w",format="ascii",mesh="twofloats:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='twofloats', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "twofloats"))
        from oof2.engine import mesh
        saved = mesh.meshes["twofloats:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)
        OOF.Microstructure.Delete(microstructure="twofloats")


    # loop of floats
    def FloatLoop(self):
        OOF.Mesh.Boundary_Conditions.New(name='bc<3>', mesh='bc_test:skeleton:mesh',
                                         condition=FloatBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='0'),boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(name='bc<4>', mesh='bc_test:skeleton:mesh',
                                         condition=FloatBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='alpha'),boundary='left'))
        OOF.Mesh.Boundary_Conditions.New(name='bc<5>', mesh='bc_test:skeleton:mesh',
                                         condition=FloatBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='0'),boundary='bottom'))
        OOF.Mesh.Boundary_Conditions.New(name='bc<6>', mesh='bc_test:skeleton:mesh',
                                         condition=FloatBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='-alpha'),boundary='right'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='floatloop')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "floatloop"),
##                            mode="w",format="ascii",mesh="floatloop:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='floatloop', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "floatloop"))
        from oof2.engine import mesh
        saved = mesh.meshes["floatloop:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)
        OOF.Microstructure.Delete(microstructure="floatloop")


    # inconsistent loop of floats
    def InconsistentFloatLoop(self):
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<3>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='0'),boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<4>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='alpha'),boundary='left'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<5>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='0'),boundary='bottom'))
        OOF.Mesh.Boundary_Conditions.New(
            name='bc<6>', mesh='bc_test:skeleton:mesh',
            condition=FloatBC(field=Temperature,field_component='',
                              equation=Heat_Eqn,eqn_component='',
                              profile=ContinuumProfile(function='alpha'),boundary='right'))
        from oof2.SWIG.common import ooferror
        # make sure an inconsistent loop raises an error
        self.assertRaises(ooferror.ErrSetupError,
            OOF.Solver.Solve,mesh="bc_test:skeleton:mesh",
                             solver=LinearDriver(
                method=GMRESSolver(max_iterations=1000,
                                   krylov_dimension=100,
                                   tolerance=1e-13,
                                   preconditioner=BlockPreconditioner(
                inplane=ILUPreconditioner()))))
        # now disable two of the boundaries to make it consistent (and
        # so that it matches the test in TwoFloats)
        OOF.Mesh.Boundary_Conditions.Disable(mesh='bc_test:skeleton:mesh',
                                             name='bc<6>')
        OOF.Mesh.Boundary_Conditions.Disable(mesh='bc_test:skeleton:mesh',
                                             name='bc<5>')
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
        OOF.File.Load.Data(filename=os.path.join("bc_data", "twofloats"))
        from oof2.engine import mesh
        saved = mesh.meshes["twofloats:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)
        OOF.Microstructure.Delete(microstructure="twofloats")
        # now re-enable the boundaries and make sure it raises the error again
        OOF.Mesh.Boundary_Conditions.Enable(mesh='bc_test:skeleton:mesh',
                                            name='bc<6>')
        OOF.Mesh.Boundary_Conditions.Enable(mesh='bc_test:skeleton:mesh',
                                            name='bc<5>')
        self.assertRaises(ooferror.ErrSetupError,
            OOF.Solver.Solve,mesh="bc_test:skeleton:mesh",
                             solver=LinearDriver(
                method=GMRESSolver(max_iterations=1000,
                                   krylov_dimension=100,
                                   tolerance=1e-13,
                                   preconditioner=BlockPreconditioner(
                inplane=ILUPreconditioner()))))


    # float intersecting fixed
    def FloatAndFixed(self):
        OOF.Mesh.Boundary_Conditions.New(name='bc<3>', mesh='bc_test:skeleton:mesh',
                                         condition=DirichletBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='0'),boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(name='bc<4>', mesh='bc_test:skeleton:mesh',
                                         condition=FloatBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='alpha'),boundary='left'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='floatandfixed')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "floatandfixed"),
##                            mode="w",format="ascii",mesh="floatandfixed:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='floatandfixed', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "floatandfixed"))
        from oof2.engine import mesh
        saved = mesh.meshes["floatandfixed:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)
        OOF.Microstructure.Delete(microstructure="floatandfixed")


    # loop of floats
    def FloatAndFixedLoop(self):
        OOF.Mesh.Boundary_Conditions.New(name='bc<3>', mesh='bc_test:skeleton:mesh',
                                         condition=DirichletBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='0'),boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(name='bc<4>', mesh='bc_test:skeleton:mesh',
                                         condition=FloatBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='alpha'),boundary='left'))
        OOF.Mesh.Boundary_Conditions.New(name='bc<5>', mesh='bc_test:skeleton:mesh',
                                         condition=FloatBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='0'),boundary='bottom'))
        OOF.Mesh.Boundary_Conditions.New(name='bc<6>', mesh='bc_test:skeleton:mesh',
                                         condition=FloatBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='-alpha'),boundary='right'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='fixedfloatloop')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "fixedfloatloop"),
##                            mode="w",format="ascii",mesh="fixedfloatloop:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='fixedfloatloop', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "fixedfloatloop"))
        from oof2.engine import mesh
        saved = mesh.meshes["fixedfloatloop:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)
        OOF.Microstructure.Delete(microstructure="fixedfloatloop")


    # fixed intersecting fixed
    def TwoFixed(self):
        OOF.Mesh.Boundary_Conditions.New(name='bc<3>', mesh='bc_test:skeleton:mesh',
                                         condition=DirichletBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='0'),boundary='top'))
        OOF.Mesh.Boundary_Conditions.New(name='bc<4>', mesh='bc_test:skeleton:mesh',
                                         condition=DirichletBC(field=Temperature,field_component='',
                                         equation=Heat_Eqn,eqn_component='',
                                         profile=ContinuumProfile(function='alpha'),boundary='left'))
        OOF.Solver.Solve(mesh="bc_test:skeleton:mesh",
                         solver=LinearDriver(
            method=GMRESSolver(max_iterations=1000,
                               krylov_dimension=100,
                               tolerance=1e-13,
                               preconditioner=BlockPreconditioner(
            inplane=ILUPreconditioner()))))
##         OOF.Microstructure.Rename(microstructure='bc_test', name='twofixed')
##         OOF.File.Save.Mesh(filename=os.path.join("bc_data", "twofixed"),
##                            mode="w",format="ascii",mesh="twofixed:skeleton:mesh")
##         OOF.Microstructure.Rename(microstructure='twofixed', name='bc_test')
        OOF.File.Load.Data(filename=os.path.join("bc_data", "twofixed"))
        from oof2.engine import mesh
        saved = mesh.meshes["twofixed:skeleton:mesh"]
        damned = mesh.meshes["bc_test:skeleton:mesh"]
        self.assertEqual(saved.compare(damned, 1.0e-13), 0)
        OOF.Microstructure.Delete(microstructure="twofixed")



        
    def tearDown(self):
        OOF.Microstructure.Delete(microstructure="bc_test")
        OOF.Property.Delete(property='Color:blue')
        OOF.Property.Delete(property='Color:white')
        OOF.Property.Delete(property='Thermal:Conductivity:Isotropic:blue_k')
        OOF.Property.Delete(property='Thermal:Conductivity:Isotropic:white_k')
        OOF.Property.Delete(property='Mechanical:Elasticity:Isotropic:blue_elastic')
        OOF.Property.Delete(property='Mechanical:Elasticity:Isotropic:white_elastic')
        OOF.Material.Delete(name='bulk')
        OOF.Material.Delete(name='boundaries')



# Routine to do regression-type testing on the items in this file.
# Tests must be run in the order they appear in the list.  This
# routine will stop after the first failure.
def run_tests():

    test_set = [
        OOF_BCTest("SimpleSolve"),
        OOF_BCTest("Periodic1"),
        OOF_BCTest("Periodic2"),
        OOF_BCTest("TwoFloats"),
        OOF_BCTest("FloatLoop"),
        OOF_BCTest("InconsistentFloatLoop"),
        OOF_BCTest("FloatAndFixed"),
        OOF_BCTest("FloatAndFixedLoop"),
        OOF_BCTest("TwoFixed")
        ]
    
    logan = unittest.TextTestRunner()
    for t in test_set:
        print "\n *** Running test: %s\n" % t.id()
        res = logan.run(t)
        if not res.wasSuccessful():
            return 0
    return 1


###################################################################
# The code below this line should be common to all testing files. #
###################################################################

if __name__=="__main__":
    # If directly run, then start oof, and run the local tests, then quit.
    import sys
    try:
        from oof2.common import oof
    except ImportError:
        print "OOF is not correctly installed on this system."
        sys.exit(4)
    sys.argv.append("--text")
    sys.argv.append("--quiet")
    sys.argv.append("--seed=17")
    oof.run(no_interp=1)
    
    success = run_tests()

    OOF.File.Quit()
    
    if success:
        print "All tests passed."
    else:
        print "Test failure."
